"use strict";
var definition_1 = require("graphql/type/definition");
var kinds_1 = require("graphql/language/kinds");
var utils_1 = require("./utils");
var pascalCase = require("pascal-case");
var typeFromAST_1 = require("graphql/utilities/typeFromAST");
exports.buildInnerModelsArray = function (schema, rootObject, flattenInnerTypes, selections, primitivesMap, appendTo, result) {
    if (result === void 0) { result = []; }
    (selections ? selections.selections : []).forEach(function (selectionNode) {
        switch (selectionNode.kind) {
            case kinds_1.FIELD:
                var fieldName = selectionNode.name.value;
                var propertyName = selectionNode.alias ? selectionNode.alias.value : fieldName;
                var field = utils_1.getFieldDef(rootObject, selectionNode);
                var rawType = field.type;
                var actualType = definition_1.getNamedType(rawType);
                if (actualType instanceof definition_1.GraphQLObjectType) {
                    var modelName = utils_1.handleNameDuplications(pascalCase(fieldName), result);
                    var model = {
                        name: modelName,
                        fields: [],
                        fragmentsUsed: [],
                        inlineFragments: [],
                        schemaTypeName: String(actualType)
                    };
                    result.push(model);
                    var resultArr_1 = result;
                    if (!flattenInnerTypes) {
                        model.innerTypes = resultArr_1 = [];
                    }
                    exports.buildInnerModelsArray(schema, actualType, flattenInnerTypes, selectionNode.selectionSet, primitivesMap, model, resultArr_1);
                    if (!appendTo) {
                        // Means we are on the root object, and we need to create the Result interface
                        appendTo = {
                            isRoot: true,
                            name: 'Result',
                            fields: [],
                            fragmentsUsed: [],
                            inlineFragments: []
                        };
                        result.push(appendTo);
                    }
                    appendTo.fields.push({
                        name: propertyName,
                        type: modelName,
                        isArray: utils_1.isArray(rawType),
                        isRequired: utils_1.isRequired(rawType)
                    });
                }
                else {
                    appendTo.fields.push({
                        name: propertyName,
                        type: utils_1.getTypeName(primitivesMap, actualType),
                        isArray: utils_1.isArray(rawType),
                        isRequired: utils_1.isRequired(rawType)
                    });
                }
                break;
            case kinds_1.FRAGMENT_SPREAD:
                var fragmentName = selectionNode.name.value;
                appendTo.fragmentsUsed.push(pascalCase(fragmentName));
                appendTo.usingFragments = appendTo.fragmentsUsed.length > 0;
                break;
            case kinds_1.INLINE_FRAGMENT:
                var root = typeFromAST_1.typeFromAST(schema, selectionNode.typeCondition);
                var schemaTypeName = selectionNode.typeCondition.name.value;
                var name_1 = schemaTypeName + 'InlineFragment';
                var fragmentModel = {
                    name: name_1,
                    fields: [],
                    fragmentsUsed: [],
                    inlineFragments: [],
                    schemaTypeName: schemaTypeName
                };
                appendTo.inlineFragments.push({
                    typeName: name_1,
                    onModel: selectionNode.typeCondition.name.value,
                });
                appendTo.hasInlineFragments = appendTo.inlineFragments.length > 0;
                result.push(fragmentModel);
                var resultArr = result;
                if (!flattenInnerTypes) {
                    fragmentModel.innerTypes = resultArr = [];
                }
                exports.buildInnerModelsArray(schema, root, flattenInnerTypes, selectionNode.selectionSet, primitivesMap, fragmentModel, resultArr);
                break;
            default:
                break;
        }
    });
    return result;
};
//# sourceMappingURL=inner-models-builer.js.map